#
# lvm_dialog_gui.py: dialog for editting a volume group request
#
# Michael Fulbright <msf@redhat.com>
#
# Copyright 2002 Red Hat, Inc.
#
# This software may be freely redistributed under the terms of the GNU
# library public license.
#
# You should have received a copy of the GNU Library Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import copy


import string

from rhpl.translate import _, N_
from rhpl.log import log

from fsset import *
from partRequests import *
from constants import *
from constants_text import *
from snack import *
from partIntfHelpers import *
import lvm


class VolumeGroupEditor:

    def numAvailableLVSlots(self):
	return max(0, lvm.MAX_LV_SLOTS - len(self.logvolreqs))

    def computeSpaceValues(self, alt_pvlist=None, usepe=None):
	if usepe is None:
	    pesize = self.peOptionMenu.current()
	else:
	    pesize = usepe

        if alt_pvlist is None:
            pvlist = self.getSelectedPhysicalVolumes(self.lvmlist.getSelection())
        else:
            pvlist = alt_pvlist
	tspace = self.computeVGSize(pvlist, pesize)
	uspace = self.computeLVSpaceNeeded(self.logvolreqs)
	fspace =  tspace - uspace

	return (tspace, uspace, fspace)

    def getPVWastedRatio(self, newpe):
        """ given a new pe value, return percentage of smallest PV wasted

        newpe - (int) new value of PE, in KB
        """
        pvlist = self.getSelectedPhysicalVolumes(self.lvmlist.getSelection())

	waste = 0.0
	for id in pvlist:
	    pvreq = self.partitions.getRequestByID(id)
	    pvsize = pvreq.getActualSize(self.partitions, self.diskset)
	    waste = max(waste, (long(pvsize*1024) % newpe)/(pvsize*1024.0))

	return waste

    def getSmallestPVSize(self):
        """ finds the smallest PV and returns its size in MB
        """
	first = 1
        pvlist = self.getSelectedPhysicalVolumes(self.lvmlist.getSelection())
	for id in pvlist:
	    pvreq = self.partitions.getRequestByID(id)
	    pvsize = pvreq.getActualSize(self.partitions, self.diskset)
	    if first:
		minpvsize = pvsize
		first = 0
	    else:
		minpvsize = min(pvsize, minpvsize)

	return minpvsize


    def reclampLV(self, newpe):
        """ given a new pe value, set logical volume sizes accordingly

        newpe - (int) new value of PE, in KB
        """

        pvlist = self.getSelectedPhysicalVolumes(self.lvmlist.getSelection())
        availSpaceMB = self.computeVGSize(pvlist, newpe)

	# see if total space is enough
        oldused = 0
        used = 0
	resize = 0
        for lv in self.logvolreqs:
            osize = lv.getActualSize(self.partitions, self.diskset)
            oldused = oldused + osize
            nsize = lvm.clampLVSizeRequest(osize, newpe, roundup=1)
	    if nsize != osize:
		resize = 1
		
            used = used + nsize

        if used > availSpaceMB:
            self.intf.messageWindow(_("Not enough space"),
                                    _("The physical extent size cannot be "
                                      "changed because otherwise the space "
                                      "required by the currently defined "
                                      "logical volumes will be increased "
                                      "to more than the available space."),
				    custom_icon="error")
            return 0

	if resize:
	    rc = self.intf.messageWindow(_("Confirm Physical Extent Change"),
					 _("This change in the value of the "
					   "physical extent will require the "
					   "sizes of the current logical "
					   "volume requests to be rounded "
					   "up in size to an integer multiple "
					   "of the "
					   "physical extent.\n\nThis change "
					   "will take affect immediately."),
					 type="custom", custom_icon="question",
					 custom_buttons=[_("Cancel"), _("Continue")])
	    if not rc:
		return 0
        
        for lv in self.logvolreqs:
            osize = lv.getActualSize(self.partitions, self.diskset)
            nsize = lvm.clampLVSizeRequest(osize, newpe, roundup=1)
            lv.setSize(nsize)

        return 1
            
    def peChangeCB(self, peOptionMenu):
        """ handle changes in the Physical Extent option menu

        peOptionMenu - the Option menu containing the items. The data value for
                       "lastval" is the previous PE value.
        """
        curval = peOptionMenu.current()

	# see if PE is too large compared to smallest PV
	# remember PE is in KB, PV size is in MB
	maxpvsize = self.getSmallestPVSize()
	if curval > maxpvsize * 1024:
            self.intf.messageWindow(_("Not enough space"),
                                    _("The physical extent size cannot be "
                                      "changed because the value selected "
				      "(%10.2f MB) is larger than the smallest "
				      "physical volume (%10.2f MB) in the "
				      "volume group.") % (curval/1024.0, maxpvsize), custom_icon="error")
	    peOptionMenu.setCurrent(self.lastval)
            return 0

	# see if new PE will make any PV useless due to overhead
	if lvm.clampPVSize(maxpvsize, curval) * 1024 < curval:
            self.intf.messageWindow(_("Not enough space"),
                                    _("The physical extent size cannot be "
                                      "changed because the value selected "
				      "(%10.2f MB) is too large compared "
                                      "to the size of the "
				      "smallest physical volume "
				      "(%10.2f MB) in the "
				      "volume group.") % (curval/1024.0,
                                                          maxpvsize),
                                    custom_icon="error")
	    peOptionMenu.setCurrent(self.lastval)
            return 0
	    

	if self.getPVWastedRatio(curval) > 0.10:
	    rc = self.intf.messageWindow(_("Too small"),
					 _("This change in the value of the "
					   "physical extent will waste "
					   "substantial space on one or more "
					   "of the physical volumes in the "
					   "volume group."),
					 type="custom", custom_icon="error",
					   custom_buttons=["gtk-cancel", _("C_ontinue")])
	    if not rc:
		peOptionMenu.setCurrent(4096)
		return 0

	# now see if we need to fixup effect PV and LV sizes based on PE
        if curval > self.lastval:
            rc = self.reclampLV(curval)
            if not rc:
		peOptionMenu.setCurrent(self.lastval)
		return 0
            else:
                self.updateLogVolStore()
	else:
	    maxlv = lvm.getMaxLVSize(curval)
	    for lv in self.logvolreqs:
		lvsize = lv.getActualSize(self.partitions, self.diskset)
		if lvsize > maxlv:
		    self.intf.messageWindow(_("Not enough space"),
					    _("The physical extent size "
					      "cannot be changed because the "
					      "resulting maximum logical "
					      "volume size (%10.2f MB) is "
					      "smaller "
					      "than one or more of the "
					      "currently defined logical "
					      "volumes.") % (maxlv,),
					    custom_icon="error")
		    peOptionMenu.setCurrent(self.lastval)
		    return 0
            
        self.updateAllowedLvmPartitionsList(self.availlvmparts,
					    self.partitions,
					    self.lvmlist)
	self.updateVGSpaceLabels()

    def clickCB(self):
	log("i am here")
        model = self.lvmlist.getSelection()
        pvlist = self.getSelectedPhysicalVolumes(model)

        # get the highlight row
        item = self.lvmlist.getCurrent()

        (availSpaceMB, neededSpaceMB, fspace) = self.computeSpaceValues(alt_pvlist=pvlist)
        if availSpaceMB < neededSpaceMB:
            self.intf.messageWindow(_("Not enough space"),
                                    _("You cannot remove this physical "
                                      "volume because otherwise the "
                                      "volume group will be too small to "
                                      "hold the currently defined logical "
                                      "volumes."), custom_icon="error")
            self.lvmlist.setEntryValue(item, selected = 1)
	    return

        self.updateVGSpaceLabels(alt_pvlist = pvlist)
        return

    def prettyFormatPESize(self, val):
        if val < 1024:
            return "%s KB" % (val,)
        elif val < 1024*1024:
            return "%s MB" % (val/1024,)
        else:
            return "%s GB" % (val/1024/1024,)

    def createPEOptionMenu(self, default=4096):
	row = 0
	peOptionMenu = Listbox(height=2, scroll=1)
	peOptionGrid = Grid(1, 2)

	actualPE = lvm.getPossiblePhysicalExtents(floor=1024)
	for curpe in actualPE:
	    # dont show PE over 64M
	    if curpe > 65536:
		continue
	    
            val = self.prettyFormatPESize(curpe)

	    peOptionMenu.append(val, curpe)

        peOptionMenu.setCurrent(default)
	peOptionGrid.setField(peOptionMenu, 0, row)
	peOptionMenu.setCallback(self.peChangeCB, peOptionMenu)
	
	return (peOptionMenu, peOptionGrid)

    def createAllowedLvmPartitionsList(self, alllvmparts, reqlvmpart, partitions, preexist = 0):

        subgrid = Grid(1, 2)
        partLbl = Label(_("Physical Volumes to Use:"))
        subgrid.setField(partLbl, 0, 0, (0,0,24,0))
        partlist = CheckboxTree(height=2, scroll=1)

	for uid, size, used in alllvmparts:
	    request = partitions.getRequestByID(uid)
	    if request.type != REQUEST_RAID:
		partname = "%s" % (request.device,)
	    else:
		partname = "md%d" % (request.raidminor,)

	    # clip size to current PE
	    pesize = self.peOptionMenu.current()
	    size = lvm.clampPVSize(size, pesize)
	    partsize = "%10.2f MB" % size
	    if used or not reqlvmpart:
		selected = 1
	    else:
		selected = 0

            if preexist == 0 or selected == 1:
		partItem = partname + " " + partsize
                partlist.append(partItem, selected = selected)

	subgrid.setField(partlist, 0, 1, (20,0,0,0))
	partlist.setCallback(self.clickCB )
	return (partlist, subgrid)

    def updateAllowedLvmPartitionsList(self, alllvmparts, partitions, partlist):
	""" update sizes in pv list

	alllvmparts - list of pv from partitions.getAvailLVMPartitions
	partitions - object holding all partition requests
	partlist - the checklist containing pv list
	"""
	for uid, size, used in alllvmparts:
            request = partitions.getRequestByID(uid)
            if request.type != REQUEST_RAID:
                partname = "%s" % (request.device,)
            else:
                partname = "md%d" % (request.raidminor,)

	    # clip size to current PE
	    pesize = self.peOptionMenu.current()
	    size = lvm.clampPVSize(size, pesize)
	    partsize = "%10.2f MB" % size

	    for item in partlist.item2key.keys():
	        l = string.split(item, "   ")
		if l[0] == partname:
		    partlist.setEntry(item, (partname + " " + partsize))
		    break;

    def getCurrentLogicalVolume(self):
	if len(self.logvolreqs) == 0:
	    return None
	selection = self.logvollist.current()
	return selection

    def makeMountEntry(self, request):
        mountgrid = Grid(2, 1)
        mountLbl = Label(_("Mount Point:"))
        mountgrid.setField(mountLbl, 0, 0, (8,0,0,0), anchorLeft = 1)
        mountpoint = request.mountpoint
        if mountpoint:
            mount = Entry(20, mountpoint)
        else:
            mount = Entry(20, "")
        mountgrid.setField(mount, 1, 0, anchorRight = 1, growx = 1)
        if request.fstype and not request.fstype.isMountable():
            mount.setFlags(FLAG_DISABLED, FLAGS_SET)
            mount.set(_("<Not Applicable>"))
        return (mount, mountgrid)

    # make the list of available filesystems and it's label
    def makeFsList(self, request, usecallback=1, uselabel=1, usetypes=None,
                   ignorefs = None):
        subgrid = Grid(1, 2)
        row = 0
        # filesystem type selection
        if uselabel:
            typeLbl = Label(_("File System type:"))
            subgrid.setField(typeLbl, 0, row, (0,0,18,0))
            row = row + 1

        fstype = Listbox(height=2, scroll=1)
        types = fileSystemTypeGetTypes()
        if usetypes:
            names = usetypes
        else:
            names = types.keys()
        names.sort()
        for name in names:
            if not fileSystemTypeGet(name).isSupported():
                continue

            if ignorefs and name in ignorefs:
                continue

            if fileSystemTypeGet(name).isFormattable():
                fstype.append(name, types[name])
	        log("The name is %s, key is %sAA", name, types[name])
        if request.fstype and request.fstype.getName() in names and \
           request.fstype.isFormattable() and request.fstype.isSupported():
            fstype.setCurrent(request.fstype)
        else:
            fstype.setCurrent(fileSystemTypeGetDefault())
        subgrid.setField(fstype, 0, row, anchorRight = 1)
        if usecallback:
            fstype.setCallback(self.fstypeSetCB, (fstype, self.mount))
        return (fstype, subgrid)

    def fstypeSet(self, obj):
        (current, entry) = obj
        flag = FLAGS_RESET
        if not current.isMountable():
            if entry.value() != _("<Not Applicable>"):
                self.oldMount = entry.value()
            entry.set(_("<Not Applicable>"))
            flag = FLAGS_SET
        elif entry.value() == _("<Not Applicable>"):
            if self.oldMount:
                entry.set(self.oldMount)
            else:
                entry.set("")

        entry.setFlags(FLAG_DISABLED, flag)

    def fstypeSetCB(self, obj):
        (listbox, entry) = obj
        self.fstypeSet((listbox.current(), entry))

    def fsOptionsDialog(self, origrequest, format, migrate, newfstype, badblocks, showbadblocks=0):

        def formatChanged((formatrb, badblocksCB)):
            flag = FLAGS_SET
            if formatrb.selected():
                flag = FLAGS_RESET

            if badblocksCB:
                badblocksCB.setFlags(FLAG_DISABLED, flag)

        poplevel = GridFormHelp(self.screen, _("File System Options"),
                                "fsoption", 1, 6)
        row = 0
        poplevel.add(TextboxReflowed(40, _("Please choose how you would "
                                           "like to prepare the file system "
                                           "on this partition.")), 0, 0)
        row = row + 1
        subgrid = Grid(2, 5)
        srow = 0

        if showbadblocks:
            badblocksCB = Checkbox(_("Check for bad blocks"))
        else:
            badblocksCB = None

        noformatrb = SingleRadioButton(_("Leave unchanged (preserve data)"),
                                       None, not format and not migrate)
        subgrid.setField(noformatrb, 0, srow, (0,0,0,1),anchorLeft = 1)

        srow = srow + 1
        if format:
            forflag = 1
        else:
            forflag = 0
        formatrb = SingleRadioButton(_("Format as:"), noformatrb, forflag)
        formatrb.setCallback(formatChanged, (formatrb, badblocksCB))
        noformatrb.setCallback(formatChanged, (formatrb, badblocksCB))

        subgrid.setField(formatrb, 0, srow, (0,0,0,1), anchorLeft = 1)

        (fortype, forgrid) = self.makeFsList(origrequest, usecallback = 0,
                                             uselabel = 0,
					     ignorefs = ["software RAID", "physical volume (LVM)", "vfat", "PPC PReP Boot"])

        if newfstype and newfstype.isFormattable() and \
           newfstype.getName() in fileSystemTypeGetTypes().keys() and \
           newfstype.isSupported():
            fortype.setCurrent(newfstype)

        subgrid.setField(forgrid, 1, srow, (0,0,0,1))

        if origrequest.origfstype and origrequest.origfstype.isMigratable():
            srow = srow + 1
            if migrate:
                migflag = 1
            else:
                migflag = 0
            migraterb = SingleRadioButton(_("Migrate to:"), formatrb, migflag)
            migraterb.setCallback(formatChanged, (formatrb, badblocksCB))
            subgrid.setField(migraterb, 0, srow, (0,0,0,1), anchorLeft = 1)

            migtypes = origrequest.origfstype.getMigratableFSTargets()

            (migtype, miggrid) = self.makeFsList(origrequest, usecallback = 0,
                                                 uselabel = 0,
                                                 usetypes = migtypes,
                                                 ignorefs = ["software RAID", "physical volume (LVM)", "vfat", "PPC PReP Boot"])

            if newfstype and newfstype.getName() in migtypes:
                migtype.setCurrent(newfstype)
            subgrid.setField(miggrid, 1, srow, (0,0,0,1))
        else:
            migraterb = None

        poplevel.add(subgrid, 0, row, (0,1,0,1))

        row = row + 1

        if badblocksCB:
            poplevel.add(badblocksCB, 0, row, (0,1,0,1))
            if badblocks:
                badblocksCB.setValue("*")
            row = row + 1

        formatChanged((formatrb, badblocksCB))

        popbb = ButtonBar(self.screen, (TEXT_OK_BUTTON, TEXT_CANCEL_BUTTON))
        poplevel.add(popbb, 0, row, (0,0,0,0), growx = 1)

        while 1:
            res = poplevel.run()

            if popbb.buttonPressed(res) == 'cancel':
                self.screen.popWindow()
                return (format, migrate, newfstype, badblocks)

            if noformatrb.selected():
                format = 0
                migrate = 0
                newfstype = origrequest.origfstype
            elif formatrb and formatrb.selected():
                format = 1
                migrate = 0
                newfstype = fortype.current()
            elif migraterb and migraterb.selected():
                format = 0
                migrate = 1
                newfstype = migtype.current()

            self.screen.popWindow()

            if badblocksCB:
                badblockstate = badblocksCB.selected()
            else:
                badblockstate = 0
            return (format, migrate, newfstype, badblockstate)

    def editLogicalVolume(self, logrequest, isNew = 0):
	if isNew:
	    tstr = _("Make Logical Volume")
	else:
	    try:
		tstr = _("Edit Logical Volume: %s") % (logrequest.logicalVolumeName,)
	    except:
		tstr = _("Edit Logical Volume")
	    
        dialog = GridFormHelp(self.screen, tstr, "editlogvol", 1, 6)

        row = 0

	(self.mount, mountgrid) = self.makeMountEntry(logrequest)
	dialog.add(mountgrid, 0, row)
        row = row + 1

        if not logrequest or not logrequest.getPreExisting():
	    (newfstype, fsgrid) = self.makeFsList(logrequest,
					   ignorefs = ["software RAID", "physical volume (LVM)", "vfat", "PPC PReP Boot"])
        else:
	    fsgrid = Grid(2, 1)
	    lbl = Label(_("Original File System Type:"))
            fsgrid.setField(lbl, 0, 0, (0,0,10,0), anchorLeft = 1)

            if logrequest.origfstype and logrequest.origfstype.getName():
		self.oldfstype = logrequest.origfstype
                newfstype = Label(logrequest.origfstype.getName())
            else:
		self.oldfstype = None
                newfstype = Label(_("Unknown"))

            fsgrid.setField(newfstype, 1, 0, (0,0,0,0), anchorLeft = 1)

	dialog.add(fsgrid, 0, row)
	row = row + 1

        # logical volume name
        namegrid = Grid(2, 1)
        nameLbl = Label(_("Logical Volume Name:"))
        namegrid.setField(nameLbl, 0, 0, (0,0,0,0), anchorLeft = 1)
        if not logrequest or not logrequest.getPreExisting():
            lvnameEntry = Entry(20, lvm.createSuggestedLVName(self.logvolreqs))
        else:
            lvnameEntry = Label(logrequest.logicalVolumeName)

        namegrid.setField(lvnameEntry, 1, 0, anchorRight = 1, growx = 1)
        dialog.add(namegrid, 0, row)
        row = row + 1
	
	sizegrid = Grid(2, 1)
	sizeLbl = Label(_("Size (MB):"))
	sizegrid.setField(sizeLbl, 0, 0, (10,0,0,0), anchorLeft = 1)
        if not logrequest or not logrequest.getPreExisting():
            if logrequest:
                sizeEntry = Entry(20, "%g" % (logrequest.getActualSize(self.partitions, self.diskset),))
        else:
            sizeEntry = Label(str(logrequest.size))

        sizegrid.setField(sizeEntry, 1, 0, anchorRight = 1, growx = 1)
        dialog.add(sizegrid, 0, row)
        row = row + 1

        if not logrequest or not logrequest.getPreExisting():
            pesize = self.peOptionMenu.current()
            (tspace, uspace, fspace) = self.computeSpaceValues(usepe=pesize)
            maxlv = min(lvm.getMaxLVSize(pesize), fspace)

            # add in size of current logical volume if it has a size
            if logrequest and not isNew:
                maxlv = maxlv + logrequest.getActualSize(self.partitions, self.diskset)
            maxlabel = Label(_("(Max size is %s MB)") % (maxlv,))
	    maxgrid = Grid(2, 1)
	    maxgrid.setField(maxlabel, 1, 0, (20,0,0,0), anchorRight = 1, growx = 1)
            dialog.add(maxgrid, 0, row)
	    row = row + 1

	self.fsoptionsDict = {}
	if not logrequest or not logrequest.getPreExisting():
	    popbb = ButtonBar(self.screen, (TEXT_OK_BUTTON, TEXT_CANCEL_BUTTON))
        else:
            popbb = ButtonBar(self.screen, (TEXT_OK_BUTTON,
                                            (_("File System Options"), "fsopts"),
                                            TEXT_CANCEL_BUTTON))


        dialog.add(popbb, 0, row)
	
	format = 0
	migrate = 0
	badblocks = 0
	while 1:
	    rc = dialog.run()

            # if the user hit cancel, do nothing
            if popbb.buttonPressed(rc) == 'cancel':
                self.screen.popWindow()
                return

            if popbb.buttonPressed(rc) == 'fsopts':
                # we do not show the badblock option any longer as it is
                # not supported.
                showbad = 0
                (format, migrate, thefstype, badblocks) = self.fsOptionsDialog(logrequest, format, migrate, self.oldfstype, badblocks, showbadblocks = showbad)
                self.fstypeSet((thefstype, self.mount))
                newfstype.setText(thefstype.getName())
		fsystem = thefstype
                continue

            if not logrequest or not logrequest.getPreExisting():
                fsystem = newfstype.current()
                format = 1
                migrate = 0
	    else:
		fsystem = self.oldfstype

            mntpt = self.mount.value() 

            if not logrequest or not logrequest.getPreExisting():
                # check size specification
                badsize = 0
                try:
                    size = long(sizeEntry.value())
                except:
                    badsize = 1

                if badsize or size <= 0:
                    self.intf.messageWindow(_("Illegal size"),
                                            _("The requested size as entered is "
                                              "not a valid number greater "
                                              "than 0."))
                    continue
            else:
                size = logrequest.size

	    # is this an existing logical volume or one we're editting
            if logrequest:
                preexist = logrequest.getPreExisting()
            else:
                preexist = 0

	    # test mount point
            # check in pending logical volume requests
	    # these may not have been put in master list of requests
	    # yet if we have not hit 'OK' for the volume group creation
	    if fsystem.isMountable():
		used = 0
		if logrequest:
		    curmntpt = logrequest.mountpoint
		else:
		    curmntpt = None
		    
		for lv in self.logvolreqs:
		    if curmntpt and lv.mountpoint == curmntpt:
			continue

		    if lv.mountpoint == mntpt:
			used = 1
			break

		if used:
		    self.intf.messageWindow(_("Mount point in use"),
					    _("The mount point \"%s\" is in "
					      "use, please pick another.") %
					    (mntpt,))
		    continue

            if not logrequest or not logrequest.getPreExisting():
	        # check out logical volumne name
	        lvname = string.strip(lvnameEntry.value())
                err = sanityCheckLogicalVolumeName(lvname)
                if err:
                    self.intf.messageWindow(_("Illegal Logical Volume Name"),err)
                    continue
	    else:
		lvname = logrequest.logicalVolumeName

	    # is it in use?
	    used = 0
	    if logrequest:
		origlvname = logrequest.logicalVolumeName
	    else:
		origlvname = None

	    for lv in self.logvolreqs:
		if origlvname and lv.logicalVolumeName == origlvname:
		    continue

		if lv.logicalVolumeName == lvname:
		    used = 1
		    break

	    if used:
		self.intf.messageWindow(_("Illegal logical volume name"),
					_("The logical volume name \"%s\" is "
					  "already in use. Please pick "
					  "another.") % (lvname,))
		continue

	    # create potential request
	    request = copy.copy(logrequest)
	    pesize = self.peOptionMenu.current()
	    size = lvm.clampLVSizeRequest(size, pesize, roundup=1)

	    # do some final tests
	    maxlv = lvm.getMaxLVSize(pesize)
	    if size > maxlv:
		self.intf.messageWindow(_("Not enough space"),
					_("The current requested size "
					  "(%10.2f MB) is larger than maximum "
					  "logical volume size (%10.2f MB). "
					  "To increase this limit you can "
					  "increase the Physical Extent size "
					  "for this Volume Group."))
		continue

 	    request.fstype = fsystem

 	    if request.fstype.isMountable():
 		request.mountpoint = mntpt
 	    else:
 		request.mountpoint = None

            request.preexist = preexist
	    request.logicalVolumeName = lvname
	    request.size = size
            request.format = format
            request.migrate = migrate
	    request.badblock = None

	    # this is needed to clear out any cached info about the device
	    # only a workaround - need to change way this is handled in
	    # partRequest.py really.
	    request.dev = None
	    
	    # make list of original logvol requests so we can skip them
	    # in tests below. We check for mount point name conflicts
	    # above within the current volume group, so it is not
	    # necessary to do now.
 	    err = request.sanityCheckRequest(self.partitions, skipMntPtExistCheck=1)
	    if err is None:
		skiplist = []
		for lv in self.origvolreqs:
		    skiplist.append(lv.uniqueID)
		    
		err = request.isMountPointInUse(self.partitions, requestSkipList=skiplist)

 	    if err:
 		self.intf.messageWindow(_("Error With Request"),
 					"%s" % (err))
 		continue

	    if (not request.format and
		request.mountpoint and request.formatByDefault()):
		if not queryNoFormatPreExisting(self.intf):
		    continue

	    # see if there is room for request
	    (availSpaceMB, neededSpaceMB, fspace) = self.computeSpaceValues(usepe=pesize)

	    tmplogreqs = []
	    for l in self.logvolreqs:
		if origlvname and l.logicalVolumeName == origlvname:
		    continue
		
		tmplogreqs.append(l)

	    tmplogreqs.append(request)
	    neededSpaceMB = self.computeLVSpaceNeeded(tmplogreqs)

	    if neededSpaceMB > availSpaceMB:
		self.intf.messageWindow(_("Not enough space"),
					_("The logical volumes you have "
					  "configured require %g MB, but the "
					  "volume group only has %g MB.  Please "
					  "either make the volume group larger "
					  "or make the logical volume(s) smaller.") % (neededSpaceMB, availSpaceMB))
		del tmplogreqs
		continue

	    # everything ok
	    break

	# now remove the previous request, insert request created above
	if not isNew:
	    item = self.getCurrentLogicalVolume()
	    if item != None:
	        self.logvollist.delete(item)
	    self.logvolreqs.remove(logrequest)
	    
        self.logvolreqs.append(request)

	if request.fstype and request.fstype.isMountable():
	    mountPoint = mntpt
	else:
	    mountPoint =  "N/A"
	log("%s", lvname)
	self.logvollist.append(["%s" %(lvname),
                                "%s" %(mountPoint),
                                "%d" %(size)], lvname,
                                [LEFT, LEFT, RIGHT])


	self.updateVGSpaceLabels()
	self.logvollist.setCurrent(lvname)
        # clean up
        self.screen.popWindow()

	
    def editCurrentLogicalVolume(self):
	logvolname = self.getCurrentLogicalVolume()

	if logvolname is None:
	    return
	
	logrequest = None
	for lv in self.logvolreqs:
	    if lv.logicalVolumeName == logvolname:
		logrequest = lv
		
	if logrequest is None:
	    return

	self.editLogicalVolume(logrequest)

    def addLogicalVolumeCB(self):
	if self.numAvailableLVSlots() < 1:
	    self.intf.messageWindow(_("No free slots"),
				    _("You cannot create more than %s logical "
				    "volumes per volume group."))
	    return
	
        (tspace, uspace, fspace) = self.computeSpaceValues()
	if fspace <= 0:
	    self.intf.messageWindow(_("No free space"),
				    _("There is no room left in the "
				      "volume group to create new logical "
				      "volumes. "
				      "To add a logical volume you will need "
				      "to reduce the size of one or more of "
				      "the currently existing "
				      "logical volumes"))
	    return
	    
        request = LogicalVolumeRequestSpec(fileSystemTypeGetDefault(),
					   size = fspace)
	self.editLogicalVolume(request, isNew = 1)
	return

    def editLogicalVolumeCB(self):
	self.editCurrentLogicalVolume()
	return

    def delLogicalVolumeCB(self):
	logvolname = self.getCurrentLogicalVolume()
	if logvolname is None:
	    return
	

	rc = self.intf.messageWindow(_("Confirm Delete"),
				_("Are you sure you want to Delete the "
				"logical volume \"%s\"?") % (logvolname,),
				type = "custom", custom_buttons=["Cancel", _("_Delete")], custom_icon="warning")

	if not rc:
    	    return

	for lv in self.logvolreqs:
	    if lv.logicalVolumeName == logvolname:
		self.logvolreqs.remove(lv)

	self.logvollist.delete(logvolname)

	self.updateVGSpaceLabels()
	return
    
    def logvolActivateCb(self, view, path, col):
	self.editCurrentLogicalVolume()

    def getSelectedPhysicalVolumes(self, model):
	pv = []
	currow = 0
	for i in model:
	    log("%s", i)
	    l = string.split(i, " ")
	    
	    if 1:
		pvreq = self.partitions.getRequestByDeviceName(l[0])
		id = pvreq.uniqueID
		pv.append(id)

	    currow = currow + 1

	return pv

    def computeVGSize(self, pvlist, curpe):
	availSpaceMB = 0
	for id in pvlist:
	    pvreq = self.partitions.getRequestByID(id)
	    pvsize = pvreq.getActualSize(self.partitions, self.diskset)
	    pvsize = lvm.clampPVSize(pvsize, curpe)

	    # have to clamp pvsize to multiple of PE
	    availSpaceMB = availSpaceMB + pvsize

	return availSpaceMB

    def computeLVSpaceNeeded(self, logreqs):
	neededSpaceMB = 0
	for lv in logreqs:
	    neededSpaceMB = neededSpaceMB + lv.getActualSize(self.partitions, self.diskset)

	return neededSpaceMB

    def updateLogVolStore(self):
	item = self.getCurrentLogicalVolume()
        self.logvollist.clear()
        for lv in self.logvolreqs:
            size = "%g" % (lv.getActualSize(self.partitions, self.diskset))
            lvname = lv.logicalVolumeName
            mntpt = lv.mountpoint
	    if lv.fstype and lv.fstype.isMountable() and mntpt:
                mountPoint = mntpt
            else:
                mountPoint = "N/A"

	    self.logvollist.append(["%s" %(lvname),
                                    "%s" %(mountPoint),
                                    "%s" %(size)], lvname,
                                    [LEFT, LEFT, RIGHT])
	if item != None:
	    self.logvollist.setCurrent(item)

    def updateVGSpaceLabels(self, alt_pvlist=None):
	if alt_pvlist == None:
	    pvlist = self.getSelectedPhysicalVolumes(self.lvmlist.getSelection())
	else:
	    pvlist = alt_pvlist
	    
        (tspace, uspace, fspace) = self.computeSpaceValues(alt_pvlist=pvlist)

	self.totalSpaceLabel.setText("%10.2f MB" % (tspace,))
	self.usedSpaceLabel.setText("%10.2f MB" % (uspace,))

	if tspace > 0:
	    usedpercent = (100.0*uspace)/tspace
	else:
	    usedpercent = 0.0
	    
	self.usedPercentLabel.setText("(%4.1f %%)" % (usedpercent,))

	self.freeSpaceLabel.setText("%10.2f MB" % (fspace,))
	if tspace > 0:
	    freepercent = (100.0*fspace)/tspace
	else:
	    freepercent = 0.0

	self.freePercentLabel.setText("(%4.1f %%)" % (freepercent,))
	self.lastval = self.peOptionMenu.current()

    def getVGSpaceLabelsValue(self, alt_pvlist=None):
	if alt_pvlist == None:
	    pvlist = self.getSelectedPhysicalVolumes(self.lvmlist.getSelection())
	else:
	    pvlist = alt_pvlist
	    
        (tspace, uspace, fspace) = self.computeSpaceValues(alt_pvlist=pvlist)

	if tspace > 0:
	    usedpercent = (100.0*uspace)/tspace
	else:
	    usedpercent = 0.0
	    
	if tspace > 0:
	    freepercent = (100.0*fspace)/tspace
	else:
	    freepercent = 0.0

	self.lastval = self.peOptionMenu.current()
	return(uspace, usedpercent, fspace, freepercent, tspace)

    # make the entry for the name of LVM volume group and it's label
    def makeNameEntry(self, request):
        namegrid = Grid(2, 1)
        nameLbl = Label(_("Volume Group Name:"))
        namegrid.setField(nameLbl, 0, 0, (0,0,0,0), anchorLeft = 1)
        namepoint = request.mountpoint
        if mountpoint:
            mount = Entry(20, mountpoint)
        else:
            mount = Entry(20, "")
        mountgrid.setField(mount, 1, 0, anchorRight = 1, growx = 1)
        if request.fstype and not request.fstype.isMountable():
            mount.setFlags(FLAG_DISABLED, FLAGS_SET)
            mount.set(_("<Not Applicable>"))
        return (mount, mountgrid)
        
    def __init__(self, partitions, diskset, screen, intf, parent, origvgrequest, isNew = 0):
	self.partitions = partitions
	self.diskset = diskset
	self.origvgrequest = origvgrequest
	self.isNew = isNew
	self.screen = screen
	self.intf = intf
	self.parent = parent
	self.result = None

        self.availlvmparts = self.partitions.getAvailLVMPartitions(self.origvgrequest,
                                                              self.diskset)
        self.logvolreqs = self.partitions.getLVMLVForVG(self.origvgrequest)
	self.origvolreqs = copy.copy(self.logvolreqs)

        # if no PV exist, raise an error message and return
        if len(self.availlvmparts) < 1:
	    self.intf.messageWindow(_("Not enough physical volumes"),
			       _("At least one unused physical "
				 "volume partition is "
				 "needed to create an LVM Volume Group.\n\n"
				 "Create a partition or RAID array "
				 "of type \"physical volume (LVM)\" and then "
				 "select the \"LVM\" option again."))
	    self.dialog = None
            return

	if isNew:
	    tstr = _("Make LVM Volume Group")
	else:
	    try:
		tstr = _("Edit LVM Volume Group: %s") % (origvgrequest.volumeGroupName,)
	    except:
		tstr = _("Edit LVM Volume Group")
	    
        poplevel = GridFormHelp(self.screen, tstr, "makelvmvolumegroup", 1, 9)

        row = 0

        # volume group name
        namegrid = Grid(2, 1)
        nameLbl = Label(_("Volume Group Name:"))
        namegrid.setField(nameLbl, 0, 0, (0,0,4,0), anchorLeft = 1)
        if not origvgrequest.getPreExisting():
            if not self.isNew:
                volnameEntry = Entry(20, self.origvgrequest.volumeGroupName)
            else:
                volnameEntry = Entry(20, lvm.createSuggestedVGName(self.partitions))
        else:
            volnameEntry = Entry(20, self.origvgrequest.volumeGroupName)
	    
        namegrid.setField(volnameEntry, 1, 0, anchorRight = 1, growx = 1)
	poplevel.add(namegrid, 0, row)
	row = row + 1

	namegrid = Grid(2, 1)
	nameLbl = Label(_("Physical Extent:"))
        namegrid.setField(nameLbl, 0, 0, (0,0,13,0), anchorLeft = 1)

        if not origvgrequest.getPreExisting():
            (self.peOptionMenu, self.peOptionGrid) = self.createPEOptionMenu(self.origvgrequest.pesize)
        else:
            # FIXME: this is a nice hack -- if we create the option menu, but
            # don't display it, getting the value always returns what we init'd
            # it to
            (self.peOptionMenu, self.peOptionGrid) = self.createPEOptionMenu(self.origvgrequest.pesize)            
	namegrid.setField(self.peOptionGrid, 1, 0, (0,0,0,0), anchorRight = 1)
	poplevel.add(namegrid, 0, row)
        row = row + 1

        (self.lvmlist, lvmlistGrid) = self.createAllowedLvmPartitionsList(self.availlvmparts, self.origvgrequest.physicalVolumes, self.partitions, origvgrequest.getPreExisting())
	poplevel.add(lvmlistGrid, 0, row)
        row = row + 1

	(uspace, usedpercent, fspace, freepercent, tspace) = self.getVGSpaceLabelsValue()
	subgrid = Grid(3, 1)
	lbl = Label(_("Used Space:"))
	subgrid.setField(lbl, 0, 0, (0,0,1,0), anchorLeft = 1)
	self.usedSpaceLabel = Label("%10.2f MB" % (uspace,))
	subgrid.setField(self.usedSpaceLabel, 1, 0, (0,0,0,0), anchorRight = 1)
	self.usedPercentLabel = Label("(%4.1f %%)" % (usedpercent,))
	subgrid.setField(self.usedPercentLabel, 2, 0, (0,0,0,0), anchorRight = 1)
	poplevel.add(subgrid, 0, row)
        row = row + 1

	subgrid = Grid(3, 1)
        lbl = Label(_("Free Space:"))
	subgrid.setField(lbl, 0, 0, (0,0,0,0), anchorLeft = 1)
	self.freeSpaceLabel = Label("%10.2f MB" % (fspace,))
	subgrid.setField(self.freeSpaceLabel, 1, 0, (0,0,0,0), anchorRight = 1)
	self.freePercentLabel = Label("(%4.1f %%)" % (freepercent,))
	subgrid.setField(self.freePercentLabel, 2, 0, (0,0,0,0), anchorRight = 1)
	poplevel.add(subgrid, 0, row)
        row = row + 1

	subgrid = Grid(2, 1)
        lbl = Label(_("Total Space:"))
	subgrid.setField(lbl, 0, 0, (0,0,10,0), anchorLeft = 1)
	self.totalSpaceLabel = Label("%10.2f MB" % (tspace,))
	subgrid.setField(self.totalSpaceLabel, 1, 0, (0,0,0,0), anchorRight = 1)
	poplevel.add(subgrid, 0, row)
        row = row + 1

	lbl = Label(_("Logical Volumes"))
	poplevel.add(lbl, 0, row, (0,0,0,0))
	row = row + 1

	# populate list of logical volumes
        self.logvollist = CListbox(height=3, cols=3,
                           col_widths=[19,11,9],
                           scroll=1, returnExit = 1,
                           width=50, col_pad=2,
                           col_labels=[_('Logical Volume Name'), _('Mount Point'), _('Size (MB)')],
                           col_label_align=[CENTER, CENTER,CENTER])
	
	if self.logvolreqs:
	    for lvrequest in self.logvolreqs:
		volumName = lvrequest.logicalVolumeName
                if lvrequest.mountpoint is not None:
		    mountPoint = lvrequest.mountpoint
		else:
		    mountPoint = "N/A"
		volumSize = "%g" % (lvrequest.getActualSize(self.partitions, self.diskset))
		self.logvollist.append(["%s" %(volumName),
				        "%s" %(mountPoint),
					"%s" %(volumSize)], volumName,
					[LEFT, LEFT, RIGHT])
	poplevel.add(self.logvollist, 0, row)
	row = row + 1

        popbb = ButtonBar(self.screen, ((_("Add"), "add"),
					(_("Edit"), "edit"),
                                        (_("Delete"), "delete"),
					TEXT_OK_BUTTON,
					TEXT_CANCEL_BUTTON))
        poplevel.add(popbb, 0, row, (0,0,0,0), growx = 1)

        while 1:
            rc = poplevel.run()
            res = popbb.buttonPressed(rc)

            if res == "add":
                self.addLogicalVolumeCB()
            elif res == "edit":
                self.editLogicalVolumeCB()
            elif res == "delete":
                self.delLogicalVolumeCB()
            elif res == "cancel":
                self.screen.popWindow()
		self.result = None
                return
	    else:
	        pvlist = self.getSelectedPhysicalVolumes(self.lvmlist.getSelection())
	        pesize = self.peOptionMenu.current()
	        availSpaceMB = self.computeVGSize(pvlist, pesize)
	        neededSpaceMB = self.computeLVSpaceNeeded(self.logvolreqs)

	        if neededSpaceMB > availSpaceMB:
		    self.intf.messageWindow(_("Not enough space"),
		  			    _("The logical volumes you have "
					      "configured require %g MB, but the "
					      "volume group only has %g MB.  Please "
					      "either make the volume group larger "
					      "or make the logical volume(s) smaller.") % (neededSpaceMB, availSpaceMB), custom_icon="error")
		    continue

	        # check volume name
	        volname = string.strip(volnameEntry.value())
	        err = sanityCheckVolumeGroupName(volname)
	        if err:
		    self.intf.messageWindow(_("Invalid Volume Group Name"), err,
					    custom_icon="error")
		    continue

	        if self.origvgrequest:
		    origvname = self.origvgrequest.volumeGroupName
	        else:
		    origname = None

	        if origvname != volname:
		    tmpreq = VolumeGroupRequestSpec(physvols = pvlist,
                                                vgname = volname)
		    if self.partitions.isVolumeGroupNameInUse(volname):
		        self.intf.messageWindow(_("Name in use"),
			  		        _("The volume group name \"%s\" is "
					          "already in use. Please pick "
					          "another." % (volname,)),
					        custom_icon="error")
		        del tmpreq
		        continue
		
		    del tmpreq

	        # get physical extent
	        pesize = self.peOptionMenu.current()

	        # everything ok
	        break

	request = VolumeGroupRequestSpec(physvols = pvlist, vgname = volname,
					 pesize = pesize)

        # if it was preexisting, it still should be
        if self.origvgrequest and self.origvgrequest.getPreExisting():
            request.preexist = 1
            
	self.result = (request, self.logvolreqs)
        self.screen.popWindow()
	return
